fogbugz_api = {}

require('http_helper')

local function debug(A, B, C, D) end

function fogbugz_api.logon(BaseUrl, Email, Password)
   local XmlSource = net.http.get{live=true, url=BaseUrl..'/api.asp',
      parameters={cmd='logon', 
         email=Email, password=Password}}
   local Xml = xml.parse{data=XmlSource}
   return Xml.response.token[1]:nodeValue()
end

function fogbugz_api.logonIfNeeded(BaseUrl, Email, Password, ExistingToken)
   local Token = ExistingToken
   if not ExistingToken then
      return fogbugz_api.logon(BaseUrl, Email, Password)
   end
   local VersionXmlSource = net.http.get{live=true, url=BaseUrl..'/api.asp', 
      parameters={cmd='version', ixWiki='1', token=Token}}
   local VersionXmlRoot = xml.parse{data=VersionXmlSource}
   if VersionXmlRoot.response and VersionXmlRoot.response.error and VersionXmlRoot.response.error.code:nodeValue() == '3' then
      return fogbugz_api.logon(BaseUrl, Email, Password)
   end
   return ExistingToken
end

-- NOTES:
-- some access, e.g. reading generated pages (pretending to be a browser),
-- requires UI-style access, authenticated with cookies
-- UI authentication uses a different mechanism - cookies sUniqueID and fb_SessionID
-- looks like we can't authenticate for both UI and API
-- this means we have to perform 2 separate authentications - oh well
--
-- RETURN VALUE:
-- Session table: { SessionId=.., UniqueId=.. }
-- for passing to e.g. readPageHtml
function fogbugz_api.uiLogon(BaseUrl, Email, Password)
   local Body, Status, Headers, Debug = net.http.post{
      url=BaseUrl..'/default.asp?', 
      body='pre=preLogon&sPerson='..Email..'&sPassword='..Password,
      debug=true, live=true, response_headers_format='multiple'}
   local UniqueId = http_helper.cookieValueByName(Headers,'sUniqueID')
   local SessionId = http_helper.cookieValueByName(Headers, 'fb_SessionId')
   local Session = { SessionId=SessionId, UniqueId=UniqueId }
   return Session
end

function fogbugz_api.uiLogonIfNeeded(BaseUrl, Email, Password, ExistingUiSession)
   if ExistingUiSession and ExistingUiSession.SessionId and ExistingUiSession.UniqueId then
      return ExistingUiSession
   end
   return fogbugz_api.uiLogon(BaseUrl, Email, Password)
end

function fogbugz_api.listArticles(BaseUrl, Token, WikiId)
   local WikiId = WikiId or '1'
   local ArticlesXmlSource = net.http.get{live=true, url=BaseUrl..'/api.asp', 
      parameters={cmd='listArticles', ixWiki=WikiId, token=Token}}
   local ArticlesXmlRoot = xml.parse{data=ArticlesXmlSource}
   return ArticlesXmlRoot
end

function fogbugz_api.readArticle(BaseUrl, PageId, Token)
   local XmlSource = net.http.get{url=BaseUrl..'/api.asp', parameters={cmd='viewArticle', token=Token, ixWikiPage=PageId}, live=true }
   --print(XmlSource)
   local XmlRoot = xml.parse{data=XmlSource}
   local Title = XmlRoot.response.wikipage.sHeadline[1]:nodeValue()
   local Body = XmlRoot.response.wikipage.sBody[1]:nodeValue()
   local Revision = XmlRoot.response.wikipage.nRevision[1]:nodeValue()
   return {Title=Title, Body=Body, Revision=Revision }
end

function fogbugz_api.uiSessionHeaders(UiSession)
   local Headers = {}
   if UiSession and UiSession.UniqueId and UiSession.SessionId then
      Headers.Cookie = 'sUniqueID='..UiSession.UniqueId..'; fb_SessionId='..UiSession.SessionId
   end
   return Headers
end

function fogbugz_api.readPageHtml(BaseUrl, PageId, UiSession) 
   local Headers = fogbugz_api.uiSessionHeaders(UiSession)
   return net.http.get{url=BaseUrl .. '/default.asp', debug=true, live=true, headers=Headers, parameters={pg='pgWiki', ixWikiPage=PageId} }
end

function fogbugz_api.readAttachment(FullLink, UiSession)
   local Headers = fogbugz_api.uiSessionHeaders(UiSession)
   local Success, Result = pcall(net.http.get, {url=FullLink, live=true, timeout=300, headers=Headers })
   if not Success then
      return nil, Result
   end
   return Result
end

function fogbugz_api.recentlyChangedPage(BaseUrl, Token, WikiId, PageIndex)
   local Param = {pg='pgWikiView', ixWiki=WikiId, sView='recently_changed'}
   -- http://fogbugz/default.asp?pg=pgWikiView&ixWiki=1&sView=recently_changed
   if PageIndex then
      Param.ixPage = PageIndex
   end
   local Headers = fogbugz_api.uiSessionHeaders(UiSession)
   local Success, Result = pcall(net.http.get, 
      {url=BaseUrl..'/default.asp', parameters=Param, debug=true, live=true, timeout=30, headers=Headers })
   if not Success then
      return nil, Result
   end
   return Result
end


function fogbugz_api.articleHistory(BaseUrl, PageId, Token)
   local XmlSource = net.http.get{url=BaseUrl..'/api.asp', parameters={cmd='listRevisions', token=Token, ixWikiPage=PageId}, live=true }
   local XmlRoot = xml.parse{data=XmlSource}
   local RevisionsNode = XmlRoot.response.revisions
   --print(RevisionsNode)
   local History = {}
   History.PageId = PageId
   History.RevisionList = {}
   for RevisionIndex = 1, RevisionsNode:childCount('revision') do
      local RevisionNode = RevisionsNode[RevisionIndex]
      local RevisionRecord = {}
      RevisionRecord.Id = RevisionNode.nRevision[1]:nodeValue()
      RevisionRecord.DateTime = RevisionNode.dt[1]:nodeValue()
      RevisionRecord.Title = RevisionNode.sTitle[1]:nodeValue()
      table.insert(History.RevisionList, RevisionRecord)
      History.LastRevision = RevisionRecord
   end
   return History
end
